@@ -57,6 +57,7 @@ clear_service_rs(list vs_group, virtual_server_t * vs, list l)
 	element e;
 	real_server_t *rs;
 	long unsigned weight_sum;
+	long signed down_threshold = vs->quorum - vs->hysteresis;
 
 	for (e = LIST_HEAD(l); e; ELEMENT_NEXT(e)) {
 		rs = ELEMENT_DATA(e);
@@ -91,7 +92,7 @@ clear_service_rs(list vs_group, virtual_server_t * vs, list l)
 			weight_sum = weigh_live_realservers(vs);
 			if (vs->quorum_state == UP && (
 				!weight_sum ||
-				weight_sum < vs->quorum - vs->hysteresis)
+				weight_sum < down_threshold)
 			) {
 				vs->quorum_state = DOWN;
 				if (vs->quorum_down) {
@@ -157,22 +158,23 @@ init_service_rs(virtual_server_t * vs)
 
 	for (e = LIST_HEAD(vs->rs); e; ELEMENT_NEXT(e)) {
 		rs = ELEMENT_DATA(e);
+		/* Do not re-add failed RS instantly on reload */
+		if (rs->reloaded) {
+			/* force re-adding of the rs into vs_group:
+			 * we may have new vsg entries */
+			if (vs->vsgname)
+				UNSET_ALIVE(rs);
+			continue;
+		}
 		/* In alpha mode, be pessimistic (or realistic?) and don't
 		 * add real servers into the VS pool. They will get there
 		 * later upon healthchecks recovery (if ever).
 		 */
 		if (vs->alpha) {
-			if (! rs->reloaded)
-				UNSET_ALIVE(rs);
+			UNSET_ALIVE(rs);
 			continue;
 		}
 		if (!ISALIVE(rs)) {
-			if (!ipvs_cmd(LVS_CMD_ADD_DEST, check_data->vs_group, vs, rs))
-				return 0;
-			else
-				SET_ALIVE(rs);
-		} else if (vs->vsgname) {
-			UNSET_ALIVE(rs);
 			if (!ipvs_cmd(LVS_CMD_ADD_DEST, check_data->vs_group, vs, rs))
 				return 0;
 			SET_ALIVE(rs);
@@ -254,15 +256,17 @@ static void
 update_quorum_state(virtual_server_t * vs)
 {
 	long unsigned weight_sum = weigh_live_realservers(vs);
+	long signed up_threshold = vs->quorum + vs->hysteresis;
+	long signed down_threshold = vs->quorum - vs->hysteresis;
 
 	/* If we have just gained quorum, it's time to consider notify_up. */
 	if (vs->quorum_state == DOWN &&
-	    weight_sum >= vs->quorum + vs->hysteresis) {
+	    weight_sum >= up_threshold) {
 		vs->quorum_state = UP;
-		log_message(LOG_INFO, "Gained quorum %lu+%lu=%lu <= %u for VS %s"
+		log_message(LOG_INFO, "Gained quorum %lu+%lu=%li <= %lu for VS %s"
 				    , vs->quorum
 				    , vs->hysteresis
-				    , vs->quorum + vs->hysteresis
+				    , up_threshold
 				    , weight_sum
 				    , FMT_VS(vs));
 		if (vs->s_svr && ISALIVE(vs->s_svr)) {
@@ -294,13 +298,13 @@ update_quorum_state(virtual_server_t * vs)
 	 */
 	if (vs->quorum_state == UP && (
 		!weight_sum ||
-	    weight_sum < vs->quorum - vs->hysteresis)
+	    weight_sum < down_threshold)
 	) {
 		vs->quorum_state = DOWN;
-		log_message(LOG_INFO, "Lost quorum %lu-%lu=%lu > %u for VS %s"
+		log_message(LOG_INFO, "Lost quorum %lu-%lu=%li > %lu for VS %s"
 				    , vs->quorum
 				    , vs->hysteresis
-				    , vs->quorum - vs->hysteresis
+				    , down_threshold
 				    , weight_sum
 				    , FMT_VS(vs));
 		if (vs->quorum_down) {
@@ -511,9 +515,8 @@ clear_diff_vsge(list old, list new, virtual_server_t * old_vs)
 	for (e = LIST_HEAD(old); e; ELEMENT_NEXT(e)) {
 		vsge = ELEMENT_DATA(e);
 		if (!vsge_exist(vsge, new)) {
-			log_message(LOG_INFO, "VS [[%s]:%d:%d:%d] in group %s no longer exist" 
-					    , inet_sockaddrtos(&vsge->addr)
-					    , ntohs(inet_sockaddrport(&vsge->addr))
+			log_message(LOG_INFO, "VS [%s:%d:%u] in group %s no longer exist"
+					    , inet_sockaddrtopair(&vsge->addr)
 					    , vsge->range
 					    , vsge->vfwmark
 					    , old_vs->vsgname);
